# -*- coding: utf-8 -*-
"""
Created on Thu Dec 15 15:35:33 2016

@author: doarni
"""
import os
from time import sleep
from colorama import init, Fore

init(autoreset=True)

i_path_ = os.path.dirname(os.path.realpath(__file__))

def return_modpath():
    os.chdir(i_path_)
    os.chdir('..')
    return os.getcwd()
    
i_path = return_modpath()
os.chdir('..')



global_file_dict = {'html_files': ['adjustment.html',
                                   'casepage.html',
                                   'login.html',
                                   'main.html',
                                   'orderpage.html',
                                   'orderpageline.html',
                                   'userpage.html'],        
                    'pyi_path_files': ['adjustStock.py',
                                      'loginPage.py',
                                      'mainPage.py',
                                      'prodChooser.py',
                                      'siteSettings.py',
                                      'stockTab.py',
                                      'stockTabMutate.py',
                                      'stockTabTable.py',
                                      'userSettings.py'],        
                    'jsoni_path_files': ['adjustStock.json',
                                        'loginPage.json',
                                        'mainPage.json',
                                        'prodChooser.json',
                                        'siteSettings.json',
                                        'stockTab.json',
                                        'stockTabMutate.json',
                                        'stockTabTable.json',
                                        'userSettings.json'],
                    'paths': {'main': i_path,
                              'excel': {'path': i_path + '\\excel',
                                         'ui': i_path + '\\excel\\ui'},
                              'html': {'path': i_path + '\\html',
                                        'pathdata': i_path + '\\html\\pathdata',
                                        'json': i_path + '\\html\\pathdata\\json'},
                              'pgsql': {'path': i_path + '\\pgsql',
                                        'sql': i_path + '\\pgsql\\sql'},
                              'req': {'path': i_path + '\\req'},
                              'ui': {'path': i_path + '\\ui'}},
                    'critical_files': {'main': ['__init__.py',
                                                'configparse.py',
                                                'hook.py',
                                                'keys.py',
                                                'livery.py',
                                                'messagebox.py',
                                                'prompts.py',
                                                'selenium.py',
                                                'chromedriver.exe'],
                                        'excel': ['loadFile.py',
                                                  'parseFile.py'],
                                        'html': ['get.py',
                                                 'build.py',
                                                 'parse.py',
                                                 'writeconfig.py'],
                                        'pgsql': ['build_library.py',
                                                  'cipher.py',
                                                  'load_data.py',
                                                  'postgres.py']}}


class Verify():
    
    def check_for_req_files():
        err_files = {}        
         
        def verFiles(_list, dir_name, key):
            os.chdir(dir_name)
            v_list = []
            unv_list = []
            list = [f for f in os.listdir(os.getcwd()) if os.path.isfile(os.path.join(os.getcwd(), f))]
            for req_f in _list:
                if req_f in list:
                    v_list.append(req_f)
                else:
                    unv_list.append(req_f)
            err_files[key] = {'unverified': unv_list, 'verified': v_list}

        os.chdir('..')
        
        verFiles(global_file_dict['html_files'], 'html', 'html')
        verFiles(global_file_dict['pyi_path_files'], 'pathdata', 'py')
        verFiles(global_file_dict['jsoni_path_files'], 'json', 'json')
        
        os.chdir(i_path_)
        
        return err_files
        
        
    def check_file_structure():
        
        v_list = []
        unv_list = []
        
        #checking for path structures
        for key,val in global_file_dict['paths'].items():
            if key == 'main':
                if os.path.exists(val) != True:
                    unv_list.append(val)
                else:
                    v_list.append(val)
            else:
                for k,v in val.items():
                    print(Fore.LIGHTYELLOW_EX +  'Verfiying ' + Fore.LIGHTCYAN_EX + '[' + Fore.LIGHTMAGENTA_EX + key + '' + Fore.LIGHTCYAN_EX + ']' + Fore.LIGHTYELLOW_EX + ' path: ' + Fore.LIGHTCYAN_EX + v)
                    sleep(0.1)

                    if os.path.exists(v) != True:
                        unv_list.append(v)
                        print(Fore.LIGHTYELLOW_EX + 'Path: ' + Fore.LIGHTCYAN_EX + v + Fore.LIGHTRED_EX + ' failed\n')
                        sleep(0.1)

                    else:
                        print(Fore.LIGHTYELLOW_EX + 'Path: ' + Fore.LIGHTCYAN_EX + v + Fore.LIGHTGREEN_EX + ' passed\n')
                        v_list.append(v)
                        sleep(0.1)

        
        return {'passed': v_list, 'failed': unv_list}
        
    def check_critical_files():
        
        crit_files_checked_out = []
        crit_files_not_checked_out = []
        
        def get_file_list(diri_path):
            return [f for f in os.listdir(diri_path) if os.path.isfile(os.path.join(diri_path, f))]
        
        def check_files(list, key):
            for file in global_file_dict['critical_files'][key]:
                if file in list:
                    print(Fore.LIGHTYELLOW_EX + 'File: ' + Fore.LIGHTCYAN_EX + file + Fore.LIGHTGREEN_EX + ' Verified')
                    print(Fore.LIGHTYELLOW_EX + 'Checking out: ' + Fore.LIGHTCYAN_EX + file)
                    sleep(0.1)
                    crit_files_checked_out.append(file)
                    print(Fore.LIGHTCYAN_EX + file + ' ' + Fore.LIGHTYELLOW_EX + 'Checked out\n')
                elif file not in list:
                    print(Fore.LIGHTYELLOW_EX + 'File: ' + Fore.LIGHTCYAN_EX + file + Fore.LIGHTRED_EX + ' Missing')
                    print(Fore.LIGHTCYAN_EX + file + ' ' + Fore.LIGHTRED_EX + 'was not checked out\n')
                    sleep(0.1)
                    crit_files_not_checked_out.append(file)
                    pass 
                
        check_files(get_file_list(global_file_dict['paths']['main']), 'main')
        check_files(get_file_list(global_file_dict['paths']['excel']['path']), 'excel')
        check_files(get_file_list(global_file_dict['paths']['html']['path']), 'html')
        check_files(get_file_list(global_file_dict['paths']['pgsql']['path']), 'pgsql')
        
        return {'checked_out': crit_files_checked_out, 'missing': crit_files_not_checked_out }
        
    def report(fdict, structdict, critdict):
        v_files = []
        unv_files = []
        
        missingi_paths = []
        vi_paths = []
        
        missing_crit_files = []
        v_crit_files = []
        
        print(Fore.LIGHTCYAN_EX + 'Verification report\n')
        sleep(0.1)

        #Print file Report
        print(Fore.LIGHTCYAN_EX + 'File Report:')
        for k, v in fdict.items():
            print('')
            if len(v['unverified']) != 0:
                print(Fore.LIGHTYELLOW_EX + 'Unverified ' + str(k) + ' files:')
                sleep(0.1)

                for f in v['unverified']:                
                    print(Fore.LIGHTRED_EX + f)
                    unv_files.append(f)
                    sleep(0.1)

            else:
                print(Fore.LIGHTYELLOW_EX + 'Verified ' + str(k) + ' files:')
                sleep(0.1)

                for f in v['verified']:        
                    v_files.append(f)
                    print(Fore.LIGHTGREEN_EX + f)
                    sleep(0.1)

        
        print('')
        #Print file struct report
        print(Fore.LIGHTCYAN_EX + 'Directory Structure Report:')
        sleep(0.1)

        for k, v in structdict.items():
            if k == 'failed':
                if len(v) != 0:
                    for fpath in v:
                        print(Fore.LIGHTRED_EX + 'Missing Path: ' + Fore.LIGHTCYAN_EX + fpath )
                        missingi_paths.append(fpath)
                        sleep(0.1)

            else:
                for fpath in v:
                    print(Fore.LIGHTYELLOW_EX + 'Path: ' + Fore.LIGHTCYAN_EX + fpath + Fore.LIGHTYELLOW_EX + ' Exists')
                    vi_paths.append(fpath)
                    sleep(0.1)

                    
        print('')
        #Print crit file report
        print(Fore.LIGHTCYAN_EX + 'Critical Files Report:')
        sleep(0.1)

        for k, v in critdict.items():
            if k == 'missing':
                if len(v) != 0:
                    for file in v:
                        print(Fore.LIGHTRED_EX + 'Missing Critical File: ' + Fore.LIGHTCYAN_EX + file )
                        missing_crit_files.append(file)
                        sleep(0.1)

            else:
                for file in v:
                    print(Fore.LIGHTYELLOW_EX + 'File: ' + Fore.LIGHTCYAN_EX + file + Fore.LIGHTYELLOW_EX + ' Exists')
                    v_crit_files.append(file)
                    sleep(0.1)

                    
        sleep(1)            
        #Cleanup report. Clean Print.
        print('\n\n')            
        print(Fore.LIGHTYELLOW_EX + 'Total Errors: ' + Fore.LIGHTRED_EX + str( len(unv_files) + len(missingi_paths) + len(missing_crit_files)))
        
        print('')
        print(Fore.LIGHTYELLOW_EX + 'Verified files: ' + Fore.LIGHTRED_EX + str( len(v_files) ))
        print(Fore.LIGHTYELLOW_EX + 'Unverified files: ' + Fore.LIGHTRED_EX + str( len(unv_files) ))

        print('')
        print(Fore.LIGHTYELLOW_EX + 'Verified Directories: ' + Fore.LIGHTRED_EX + str( len(vi_paths) ))
        print(Fore.LIGHTYELLOW_EX + 'Missing Directories: ' + Fore.LIGHTRED_EX + str( len(missingi_paths) ))

        print('')
        print(Fore.LIGHTYELLOW_EX + 'Verified Critical Files: ' + Fore.LIGHTRED_EX + str( len(v_crit_files) ))
        print(Fore.LIGHTYELLOW_EX + 'Missing Critical Files: ' + Fore.LIGHTRED_EX + str( len(missing_crit_files) ))
        
        print('')
        if len(missing_crit_files) != 0:
            print(Fore.LIGHTRED_EX + 'These files are missing and are critical to this programs functionality:')
            for file in missing_crit_files:
                print(Fore.LIGHTMAGENTA_EX + file)
        
        print('\n\n')            

def report():
    os.chdir(i_path_)
    fdict = Verify.check_for_req_files()
    structdict = Verify.check_file_structure()
    critdict = Verify.check_critical_files()
    
    Verify.report(fdict, structdict, critdict)
    
if __name__ == '__main__':
    
    
    
    import argparse
    
    parser = argparse.ArgumentParser(description = 'Build program latest version')
    parser.add_argument('-v','--verify', action='store_true', help='Verify file Structure', required=False)

    args = vars(parser.parse_args())
    
    if args['verify']:
        os.chdir(i_path_)
        
        report()




# -*- coding: utf-8 -*-
"""
Created on Mon Nov 28 14:35:05 2016

@author: doarni
"""

import os
import json

try:
    from build import buildPaths
except:
    from pySMS.html.build import buildPaths


#===============================================================================================

class writer():
   
    def __init__(self):
        self.path = os.path.dirname(os.path.realpath(__file__))
        self.bl = buildPaths.buildLoginPaths()
        self.bm = buildPaths.buildMainPaths()
        self.bs = buildPaths.buildStockTab()
        self.bu = buildPaths.buildUserPage()
        self.ba = buildPaths.buildAdjustments()
        self.dict = {}

    #CSS Path builders, called when correct page html is saved. Return dicts of paths
    def loginPage(self):
        dict = {'url': "http://www.zimmersms.com",
                'username': self.bl.bUsername(),
                'password': self.bl.bPassword(),
                'loginbutton': self.bl.bLoginButton()}
        return dict
    def mainPage(self):
        dict = {'CaseTab': self.bm.bCaseTab(),
                'InventoryTab': self.bm.bInventoryTab(),
                'checkSite': self.bm.bCheckSite()}
        return dict
    def prodChooserPage(self):
        dict = {'prodchooserSelect': self.bs.pcSelect(),
                'prodchooserSearchButton': self.bs.pcSearchButton(),
                'prodchooserSearchBar': self.bs.pcSearchBar(),
                'prodchooserFinishButton': self.bs.pcFinishButton(),
                'prodchooserAdd': self.bs.pcAdd(),
                'prodChooserAddLastFront': self.bs.pcAddLast()[0],
                'prodChooserAddLastBack': self.bs.pcAddLast()[1],
                'prodChooserAddFront': self.bs.pcAddFront(),
                'prodChooserAddOddBack': self.bs.pcAddOddBack(),
                'prodChooserAddEvenBack': self.bs.pcAddEvenBack(),
                'prodChooserDescFront': self.bs.pcTableData()[0],
                'prodChooserDescBack': self.bs.pcTableData()[1],
                'prodChooserResults': self.bs.pcResults(),
                'prodChooserTableData': self.bs.pcTable(),
                'prodChooserTRpathFront': self.bs.pcTRPaths()[0],
                'prodChooserTRpathBack': self.bs.pcTRPaths()[1]}

        return dict

    def stockTab(self):
        dict = {'stStockTabButton': self.bs.stStockTabButton(),
                'stStockTabSearchButton': self.bs.stStockTabSearchButton(),
                'stStockTabResults': self.bs.stStockTabResults()}        
        return dict
        
    def stockTabTable(self):
        d = self.bs.stTableData()
        dict = {'stTableProductFront': d['product'][0],
                'stTableProductBack': d['product'][1],
                'stTableDescriptionFront': d['description'][0],
                'stTableDescriptionBack': d['description'][1],
                'stTableLotSerialFront': d['lot_serial'][0],
                'stTableLotSerialBack': d['lot_serial'][1],
                'stTableQOHFront': d['QOH'][0],
                'stTableQOHBack': d['QOH'][1],
                'stTableQAVFront': d['QAV'][0],
                'stTableQAVBack': d['QAV'][1],
                'stTableQRFront': d['QR'][0],
                'stTableQRBack': d['QR'][1],
                'stTableContainerInit': d['contained_in']['init'],
                'stTableContainerFront': d['contained_in']['front'],
                'stTableContainerBackInfo': d['contained_in']['backInfo'],
                'stTableContainerBackType': d['contained_in']['backType'],
                'stTableLocationFront': d['location']['front'],
                'stTableLocationBackInfo': d['location']['backInfo'],
                'stTableLocationBackLocType': d['location']['backLocType'],
                'stTableZSMSHoldInit': d['ZSMS_hold']['init'],
                'stTableZSMSHoldFront': d['ZSMS_hold']['front'],
                'stTableZSMSHoldBackEven': d['ZSMS_hold']['backEven'],
                'stTableZSMSHoldBackOdd': d['ZSMS_hold']['backOdd'],
                'stTableCheckBoxInit': self.bs.stTable_CheckBox_Init(),
                'stTableCheckBoxRowEvenFront': self.bs.stTable_CheckBox_Row_Even()[0],
                'stTableCheckBoxRowEvenBack': self.bs.stTable_CheckBox_Row_Even()[1],
                'stTableCheckBoxRowOddFront': self.bs.stTable_CheckBox_Row_Odd()[0],
                'stTableCheckBoxRowOddBack': self.bs.stTable_CheckBox_Row_Odd()[1]}
        
        return dict
    
    def stockTableMutate(self):
        d = self.bs.stMutate()
        dict = {'stMutateButton': d['mutateButton'],
                'stMutateQuantity': d['mutateQuantity'],
                'stMutateNote': d['mutateNote'],
                'stMutateAllAvailable': d['mutateAllAvailable'],
                'stMutateSavebutton': d['mutateSaveButton'],
                'stMutateCancelButton': d['mutateCancelButton'],
                'stMutateAlert': d['mutateAlert'],
                'stMutateError': d['mutateError'],
                'stnote': 'CI-ZDI Mutation, Loaners Late Debit Policy'}  
        return dict

    def userSettings(self):
        d = self.bu.uUserSettingsMenu()
        dictMain = {}
        dictMain['upUser'] = self.bu.uUser()
        dictMain['upSaveButton'] = d['saveButton']
        dictMain['upUserSettingsButton'] = self.bu.uUserSettingsButton()
        return dictMain

    def userSettingsSites(self):
        d = self.bu.uUserSettingsMenu()
        return d

    def adjustStock(self):  
        dict = {'adjAdjustButton': self.ba.adjAdjustButton(),
                'adjMenuQuantity': self.ba.adjMenuQuantity(),
                'adjMenuNote': self.ba.adjMenuNote(),
                'adjMenuCancelButton': self.ba.adjMenuCancelButton(),
                'adjMenuSaveButton': self.ba.adjMenuSaveButton()}
        
        return dict
    #Config writers, make py files and ini files    
                              
    def writeJsonFromListOfDict(self, list, filename):
        with open(self.path + '\\pathdata\\json\\' + filename + '.json', 'w') as fp:
            for dict in list:
                json.dump(dict, fp, sort_keys=True, indent=4)                

    def writeConfigFromListOfDict(self, list, filename):
        with open(self.path + '\\pathdata\\' + filename + '.py', 'w') as configfile:
            for dict in list:
                for key, value in dict.items():
                    print('Writing Key: ' + key + ' Value: {!s} \n'.format(str(value)))
                    configfile.write(key + ' = ' + "'" + value + "'" + '\n')
                
    def writeConfigWithDictionary(self, list):
        with open('htmlpaths.py', 'w') as configfile:
            for dict in list:
                for key, value in dict.items(): 
                    print('Writing Key: ' + key + ' Value: {!s} \n'.format(str(value)))
                    configfile.write(key + ' = ' + "'" + value + "'" + '\n')
 
#===============================================================================================


w = writer()

def makeloginPageConfig():
    list = []
    print('Preparing Login Config \n')
    list.append(w.loginPage())
    w.writeConfigFromListOfDict(list, 'loginPage')
    w.writeJsonFromListOfDict(list, 'loginPage')

def makemainPageConfig():
    list = []
    print('Preparing Main Page Config \n')
    list.append(w.mainPage())
    w.writeConfigFromListOfDict(list, 'mainPage')
    w.writeJsonFromListOfDict(list, 'mainPage')


def makeprodChooserPageConfig():
    list = []
    print('Preparing ProdChooser Config \n')
    list.append(w.prodChooserPage())
    w.writeConfigFromListOfDict(list, 'prodChooser')
    w.writeJsonFromListOfDict(list, 'prodChooser')

def makestockTabTableConfig():
    list = []
    print('Preparing Stock Tab Table Config \n')
    list.append(w.stockTabTable())    
    w.writeConfigFromListOfDict(list, 'stockTabTable')
    w.writeJsonFromListOfDict(list, 'stockTabTable')


def makestockTableMutateConfig():
    list = []
    print('Preparing Mutation Config \n')
    list.append(w.stockTableMutate())  
    w.writeConfigFromListOfDict(list, 'stockTabMutate')
    w.writeJsonFromListOfDict(list, 'stockTabMutate')


def makestockTabConfig():
    list = []
    print('Preparing Stock Tab Config \n')
    list.append(w.stockTab())  
    w.writeConfigFromListOfDict(list, 'stockTab')
    w.writeJsonFromListOfDict(list, 'stockTab')


def makeuserSettingsConfig():
    list = []
    print('Preparing User Settings Config \n')
    list.append(w.userSettings())
    w.writeConfigFromListOfDict(list, 'userSettings')
    w.writeJsonFromListOfDict(list, 'userSettings')

 
def makeuserSettingsSitesConfig():
    list = []
    list.append(w.userSettingsSites())
    w.writeJsonFromListOfDict(list, 'siteSettings')

  
def makeadjustStockConfig():
    list = []
    print('Preparing Adjustments Config \n')
    list.append(w.adjustStock())
    w.writeConfigFromListOfDict(list, 'adjustStock')
    w.writeJsonFromListOfDict(list, 'adjustStock')



#===============================================================================================

    
if __name__ == '__main__':
    None
#    makeadjustStockConfig()
#    makeprodChooserPageConfig()
#    makestockTableMutateConfig()
#    makestockTabConfig()
#    makeuserSettingsConfig()
    makeloginPageConfig()
#    makeuserSettingsSitesConfig()


# -*- coding: utf-8 -*-
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser  # ver. < 3.0
import os

class cfg():
        
    def configparse(section, a, b):
        config = ConfigParser()
        file = os.path.expanduser(os.sep.join(["~","pysms","pysms","html","htmlConfig.cfg"]))
        config.read(file)
        string = config.items(section)[a][b]
        return string
        
    def configwriter(dictMain, fname, flocation):
        config = ConfigParser()
        for key, val in dictMain.items():
            config[key] = val
            with open(flocation + '\\' + fname, 'w')as configfile:
                config.write(configfile)
        


# -*- coding: utf-8 -*-
"""
Created on Tue Nov 22 10:12:16 2016

@author: doarni
"""


from tkinter import *


class widget():
    
    def __init__(self):
        self.val = None
         
    def entryBox(self, txt, hide=""):
        root = Tk()
        label1 = Label( root, text=txt)
        E1 = Entry(root, bd =5)
        E1.config(show=hide)
        def getStuff():
            self.val = (E1.get())
            root.destroy()
        submit = Button(root, text ="Ok", command = getStuff)
        submit.bind('<Return>', getStuff)
        label1.pack()
        E1.pack()
        E1.focus()
        submit.pack(side =BOTTOM) 

        root.call('wm', 'attributes', '.', '-topmost', True)
        root.after_idle(root.call, 'wm', 'attributes', '.', '-topmost', False)
        root.focus_force()     
        
        root.mainloop()
        
    
    def get(self, text, char=""):
        self.val = None
        self.entryBox(text, char)
        return self.val
        



from setuptools import setup

setup(name='pySMS',
        	version= '3.0.0',
    description='ZSMS Python Module',
    author='Ian Doarn',
    author_email='ian.doarn@zimmerbiomet.com',
    url='https://github.com/IanDoarn/pySMS',
    packages=['pySMS',
              'pySMS.utils',
              'pySMS.utils.logic',
              'pySMS.data',
              'pySMS.json',
              'pySMS.pgsql',
              'pySMS.pgsql.sql',
              'pySMS.excel',
              'pySMS.excel.ui',
              'pySMS.ui',
              'pySMS.req',
              'pySMS.html',
              'pySMS.html.pathdata',
              'pySMS.html.pathdata.json'],
    install_requires=['selenium>=3.0.2',
                      'lxml>=3.7.2',
                      'colorama>=0.3.7',
                      'prettytable>=0.7.2',
                      'cssselect>=1.0.1',
                      'pythonnet>=2.2.2'],
    package_data={'pySMS.html.pathdata.json': ['*.json'],
                  'pySMS.html': ['*.html'],
                  'pySMS.pgsql.sql': ['*.sql'],
                  'pySMS.data': ['*.json.gz', '*.gz'],
                  'pySMS.json': ['*.json'],
                  'pySMS': ['*.exe', '*.ini', '*.pyd', '*.dll', '*.xml']})


stStockTabSearchButton = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td:nth-child(1) > span > div > div.GLNOMTPDJ3C.GLNOMTPDG3C'
stStockTabResults = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(2) > div > table > tbody > tr > td:nth-child(1) > div > table > tbody > tr > td > table > tbody > tr > td.GLNOMTPDBAE'
stStockTabButton = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td:nth-child(2) > table > tbody > tr > td:nth-child(1) > table > tbody > tr > td:nth-child(2) > div'



# -*- coding: utf-8 -*-
"""
Created on Wed Dec 28 07:33:37 2016

@author: doarni
"""

import json
import os
import time
import gzip
import getpass
import datetime
import pySMS.pgsql.load_data as load_data

from colorama import init, Fore
init(autoreset=True)


class structData():
    
    def __init__(self):
        self.path = os.path.dirname(os.path.realpath(__file__))
        self.db = load_data.data_builder()
        self.dict = None
        self.total_kits = 0
        self.total_tray = 0
        self.total_components = 0
        self.total_pieces = 0

    def get_time_elapsed(self, start_time):  # elapsed time
        return str(datetime.timedelta(seconds=(time.time() - start_time)))

    def get_current_time(self):
        return time.time()
        
    def check_for_old_library(self):
        if os.path.isfile(os.sep.join(["C:", "Users", getpass.getuser(),  "Desktop"]) + '\\ci_data_library.json')  == True:
            print(Fore.LIGHTRED_EX + 'A previous library has been found. ' + os.sep.join(["C:", "Users", getpass.getuser(),  "Desktop"]) + '\\ci_data_library.json')
            print(Fore.LIGHTRED_EX + 'Would you like to remove this library?')

            if input('Y/N: ').upper() == 'Y':
                os.system('del /s /q ' + os.sep.join(["C:", "Users", getpass.getuser(),  "Desktop"]) + '\\ci_data_library.json')
            else:
                print(Fore.LIGHTRED_EX + 'ci_data_library.json will be overwritten.')
        
    def close_conn(self):
        self.db.close_pgsql()
        
    def load_data(self):
        self.db.load_pgsql()
        self.db.load_ci_data()
        self.dict = self.db.return_ci_data()

    def build_library(self): #build collection of product type specific dictionaries
        _ids = {'legacy_type': 0,'item_id': 5, 'product_number': 2, 'edi_number': 3, 'description': 4, 'inv_type': 1}
        dict = self.dict  
        library = {}

        print(Fore.LIGHTCYAN_EX + 'Building ci_data_library.json')

        start_time = self.get_current_time()

        for key, val in dict.items():
            print('')    
            print(Fore.LIGHTYELLOW_EX + 'Building book: ' + key)  
            book = {}
            pages = {}
            print(Fore.LIGHTYELLOW_EX + 'Writing book: ' + key)
            for item in val:
                print(Fore.LIGHTCYAN_EX + 'With book ' + key + Fore.LIGHTYELLOW_EX +
                      Fore.LIGHTYELLOW_EX + ' Writing page: ' + Fore.CYAN + str(item[_ids['product_number']]) +
                      Fore.LIGHTYELLOW_EX + ' Legacy Type: ' + Fore.CYAN + str(item[_ids['legacy_type']]) +
                      Fore.LIGHTYELLOW_EX + ' Description: ' + Fore.CYAN + str(item[_ids['description']]))
                pages[str(item[_ids['product_number']])] = {'data_parent': key,
                                                            'inv_type': str(item[_ids['inv_type']]),
                                                            'item_id': str(item[_ids['item_id']]),
                                                            'legacy_type': item[_ids['legacy_type']],
                                                            'edi_number': item[_ids['edi_number']],
                                                            'description': item[_ids['description']]}
            
            book[key] = pages
            print('')
            print(Fore.LIGHTYELLOW_EX + 'Adding book: ' + Fore.CYAN + key + Fore.LIGHTYELLOW_EX + ' to library')
            library[key + '_book'] = book
            time.sleep(3)

        print('')
        print('')
        desktop = os.sep.join(["C:", "Users", getpass.getuser(),  "Desktop"])
        print(Fore.LIGHTGREEN_EX + 'Library Complete.')
        print('Writing ci_data_library.json')
        with open(desktop + '\\ci_data_library.json', 'w')as fp:
            json.dump(library, fp, sort_keys=False, indent=5)

        print(Fore.LIGHTGREEN_EX + 'Converting library.')
        print('Writing ci_data_library.json.gz')

        data_obj = json.dumps(library).encode('utf-8')

        with gzip.open(desktop + '\\ci_data_library.json.gz', 'wb')as f:
            f.write(data_obj)

        print(Fore.LIGHTGREEN_EX + 'Complete.')

        print(Fore.LIGHTYELLOW_EX + "Library Build Report:")

        total_time = self.get_time_elapsed(start_time)

        print(Fore.LIGHTYELLOW_EX + "Total elapsed time: " + Fore.LIGHTMAGENTA_EX + str(total_time))



def main():
    sD = structData()
    sD.check_for_old_library()

    sD.load_data()
    sD.build_library()
    sD.close_conn()
    
if __name__ == '__main__':
    main()    



class GenericError(Exception):
    """Basic Exception for modules"""
    def __init__(self, func, msg=None):
        if msg is None:
            print("An error occurred on the function: " + func)
        super(GenericError, self).__init__(msg)
        self.func = func


class NoRecordsFoundError(Exception):
    """No Record Found while parsing"""
    def __init__(self, location, msg=None):
        if msg is None:
            print("No Records Found: no records found while parsing: " + location)
        super(NoRecordsFoundError, self).__init__(msg)
        self.location = location


class ProductChooserError(Exception):
    """Called when there is an issue using Product Chooser"""
    def __init__(self, problem, msg=None):
        if msg is None:
            print("An error occurred with the product chooser: " + problem)
        super(ProductChooserError, self).__init__(msg)
        self.problem = problem


class ProductChooserAddError(Exception):
    """Called when there is an issue using Product Chooser"""
    def __init__(self, problem, msg=None):
        if msg is None:
            print("An error occurred with the product chooser" + problem)
        super(ProductChooserAddError, self).__init__(msg)
        self.problem = problem


class LogicError(Exception):
    """Basic exception for Logic Module"""
    def __init__(self, func, msg=None):
        if msg is None:
            print("An error occurred on the function: " + func)
        super(LogicError, self).__init__(msg)
        self.func = func


class DataRetreivalError(Exception):
    """Basic exception for Logic Module when data cannot be properly pulled and or read from a dict"""
    def __init__(self, func, msg=None):
        if msg is None:
            print("Could not read data from dict.")
        super(DataRetreivalError, self).__init__(msg)
        self.func = func


class DataLibError(Exception):
    """Basic exception for when data was either not retrieved from or could not be found in the ci_data_library.json file"""
    def __init__(self, item=None, msg=None):
        if item is None:
            print("Could not find item in the current data set. Please rebuild the data library.")
        elif item is not None:
            if msg is None:
                print("Could not find [{}] in the current data set. Please rebuild the data library.". format(item))
            if msg is not None:
                print("Could not find [{}] in the current data set. {}".format(item, msg))
        super(DataLibError, self).__init__(item, msg)
        self.item = item


class InvalidItemError(Exception):
    """Called when item cannot be found in data set"""
    def __init__(self, item, msg=None):
        if msg is None:
            print(item + " > Cannot be found in the supplied data set. Item may not be spelled correctly, data is corrupt, or item does not exist")
        super(InvalidItemError, self).__init__(msg)
        self.item = item


class SelectionError(Exception):
    """Basic Selection Exception for modules"""
    def __init__(self, func, element, msg=None):
        if msg is None:
            print("Unable to interact with element:  " + element + " : " + func)
        super(SelectionError, self).__init__(msg)
        self.func = func
        self.element = element


class MissingInformationError(Exception):
    """Called when information is missing from ini file"""
    def __init__(self, section, key_list, msg=None):
        if msg is None:
            print("Required information is missing. Section [{}] Keys: {}".format(section.upper(), str(key_list)))
        if msg is not None:
            print(msg + " Section [{}]: Keys: {}".format(section.upper(), str(key_list)))
            input('Press any key to continue...')
        super(MissingInformationError, self).__init__(msg)
        self.section = section
        self.key_list = key_list


class WaitTimeOutError(Exception):
    """Called when time overflows"""
    def __init__(self, n, selector, msg=None):
        if msg is None:
            print("Function timed out after [" + n + "] iterations on selector [" + selector + "]")
        super(WaitTimeOutError, self).__init__(msg)
        self.n = n
        self.selector = selector


class NetworkDriveError(Exception):
    '''Called when there is an issue with the mapped network drive'''
    def __init__(self, resolution, issue='NetworkDriveError', msg=None):
        if msg is None:
            print(issue + ': An issue occurred when connecting to the network drive. ' + resolution)
        else:
            print(issue + ': An issue occurred when connecting to the network drive. ' + resolution)
        super(NetworkDriveError, self).__init__(msg)
        self.resolution = resolution


class NetworkError(Exception):
    '''Called when there is an issue with the vpn / server connection'''
    def __init__(self, resolution, issue='NetworkConnectionError', msg=None):
        if msg is None:
            print(issue + ': Unable to connect to the server. ' + resolution)
        else:
            print(issue + ': ' + msg + '. ' + resolution)
        super(NetworkDriveError, self).__init__(msg)
        self.resolution = resolution


InventoryTab = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div > div > table > tbody > tr > td.GLNOMTPDKX > img:nth-child(2)'
CaseTab = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div > div > table > tbody > tr > td.GLNOMTPDKX > img:nth-child(1)'
checkSite = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div > div > table > tbody > tr > td.GLNOMTPDJX > table > tbody > tr > td.GLNOMTPDFX > span'



adjMenuSaveButton = 'body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div.GLNOMTPDNBD > div > table > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > span > div > div.GLNOMTPDN4C.GLNOMTPDK4C'
adjAdjustButton = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(5) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td.GLNOMTPDNWG > table > tbody > tr > td.GLNOMTPDMWG > span'
adjMenuQuantity = 'body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div.GLNOMTPDNBD > div > table > tbody > tr:nth-child(1) > td > table > tbody > tr:nth-child(1) > td.GLNOMTPDDS > input'
adjMenuCancelButton = 'body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div.GLNOMTPDNBD > div > table > tbody > tr:nth-child(2) > td > table > tbody > tr > td.GLNOMTPDBYD > span > div > div.GLNOMTPDJ3C.GLNOMTPDG3C'
adjMenuNote = 'body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div.GLNOMTPDNBD > div > table > tbody > tr:nth-child(1) > td > table > tbody > tr:nth-child(2) > td.GLNOMTPDDS > textarea'



# -*- coding: utf-8 -*-
"""
Created on Thu Dec 29 09:53:07 2016

@author: doarni
"""

import subprocess
import importlib

class Prereq():
    
    def __init__(self):
        self.req_mod_list = ['selenium',
                             'colorama',
                             'binascii',
                             'lxml',
                             'xlrd',
                             'prettytable',
                             'cssselect',
                             'pythoncom',
                             'psycopg2']
                                 
    def importer(self):
        print('Checking for required modules')
        list = []
        for module in self.req_mod_list:
            try:
                print('Checking for ' + module)
                importlib.import_module(module)
                print(module + ' imported successfully')
            except:
                print('Unable to import ' + module)
                list.append(module)
        #self.install(list)
        print('Unable to locate the following modules:')
        i = 1
        for item in list:
            print(str(i) + ") " + item)

        if input('Install them now? Y/N: ').upper() == 'Y':
            self.install(list)
        else:
            pass

    def install(self, list):
        for item in list:
            subprocess.call(['pip','install',item])
 
    def update(self):
        for item in self.req_mod_list:
            subprocess.call(['pip','install',item,'--upgrade'])
     
def import_pkgs():
    pr = Prereq()
    pr.importer()
    
def upgrade_pkgs():
    pr = Prereq()
    pr.update()    
    
if __name__ == '__main__':
    import argparse
    
    parser = argparse.ArgumentParser(description = 'Build program latest version')
    parser.add_argument('-up','--update', action='store_true', help='Update / Install Required Modules', required=False)
    parser.add_argument('-ug','--upgrade', action='store_true', help='Update Required Modules', required=False)

    args = vars(parser.parse_args())
    
    if args['update']:      
        import_pkgs()
    
    if args['upgrade']:      
        upgrade_pkgs()
        